<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */

namespace FiloBlu\Esb\Model\Rest;
use Magento\Framework\Phrase;
use FiloBlu\Esb\Api\ExternalInvoiceInterface;
use Magento\Sales\Api\Data\InvoiceInterface;
use Magento\Framework\Filesystem\DriverPool;
use Magento\Framework\App\Filesystem\DirectoryList;

if (!defined("DS")):
      define("DS", DIRECTORY_SEPARATOR);
endif;

/**
 * REST Api to save external invoices
 */
class ExternalInvoice implements ExternalInvoiceInterface{
	
    /**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 	
    protected $_orderRepository;
	
    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder 
     */ 
    protected $_searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 
    protected $_orderExtensionFactory;

	/**
     * @var \Magento\Framework\Filesystem
     */ 
    protected $_filesystem;

    /**
     * @var \Monolog\Logger
     */ 
    protected $_logger;

    /**
     * @var \FiloBlu\Esb\Api\Data\AcknowledgeInterface
     */     
    protected $_response;
    
    /**
     * Init plugin
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder     
     */
    public function __construct(
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \Magento\Sales\Model\OrderRepository $orderRepository,
            \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
            \Magento\Framework\Filesystem $filesystem,
            \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
            \FiloBlu\Esb\Helper\LoggerProvider $loggerProvider,
            \FiloBlu\Esb\Api\Data\AcknowledgeInterface $response
    ){
        $this->_scopeConfig = $scopeConfig;
        $this->_orderRepository = $orderRepository;
        $this->_orderExtensionFactory = $orderExtensionFactory;
        $this->_filesystem = $filesystem;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;              
        $this->_logger=$loggerProvider->getLogger();
        $this->_response = $response;
    }

    /**
     * Save External Invoice data. Save PDF file and additional data on Sales Order.
     *
     * @param \Magento\Sales\Api\Data\InvoiceInterface  $invoice_data
     * @return int
     */
     public function save(InvoiceInterface $invoice_data){
        $invoiceExtensionAttributes=$invoice_data->getExtensionAttributes();
        
        if(null==$invoiceExtensionAttributes || !$invoiceExtensionAttributes->getOrderIncrementId() || !$invoiceExtensionAttributes->getDocumentType()){
            $message=__("Cannot find order to save external document");
            $this->_logger->critical($message->render());
            throw new Magento\Sales\Exception\CouldNotInvoiceException(new Phrase($message->render()));
        }
        if(null==$invoice_data->getIncrementId()){
            $message=__("Cannot find order to save external document");
            $this->_logger->critical($message->render());
            throw new Magento\Sales\Exception\CouldNotInvoiceException(new Phrase($message->render()));
        }
        $orderIncrementId=$invoiceExtensionAttributes->getOrderIncrementId();
        $invoiceIncrementId=$invoice_data->getIncrementId();
        $documentType=$invoiceExtensionAttributes->getDocumentType();
        $filename="order_".$orderIncrementId."_".$documentType."_".$invoiceIncrementId.".pdf";
        $filename=preg_replace("/[^0-9A-Za-z.]+/","_",$filename );
        $documentFolder = $this->_scopeConfig->getValue(
            'esb/print_documents/document_folder'
        );              
        $writer = $this->_filesystem->getDirectoryWrite(DirectoryList::VAR_DIR);
        /** @var \Magento\Framework\Filesystem\File\WriteInterface|\Magento\Framework\Filesystem\File\Write $file */
        
        $file = $writer->openFile($documentFolder.DS.$filename, 'w');
        try {
            $file->lock();
            try {
                $file->write(base64_decode($invoiceExtensionAttributes->getDocumentPdf()));
            }
            finally {
                $file->unlock();
            }
        }
        finally {
            $file->close();
        }

        $invoice=array(
            "increment_id" => $invoice_data->getIncrementId(),
            "document_type" => $invoiceExtensionAttributes->getDocumentType(),
            "grand_total" => $invoice_data->getGrandTotal(),
            "document_pdf" => $filename
        );


        $searchCriteria = $this->_searchCriteriaBuilder
            ->addFilter('increment_id', $invoiceExtensionAttributes->getOrderIncrementId(), 'eq')->create();
        $orderList = $this->_orderRepository->getList($searchCriteria);
        
        if(count($orderList)<>1){
            $message=__("Cannot find order %1", $invoiceExtensionAttributes->getOrderIncrementId());
            $this->_logger->critical($message->render());
            throw new Magento\Sales\Exception\CouldNotInvoiceException(new Phrase($message->render()));
        }
        try{
            $order=$orderList->getFirstItem();
            $original_status=$order->getStatus();
            $orderExtensionAttributes=$order->getExtensionAttributes();
            $additionalData=array("external_invoices" => array($invoice_data->getIncrementId() => $invoice));
            if($orderExtensionAttributes){
                if($orderExtensionAttributes->getFbAdditionalData()){
                    $additionalData=current($orderExtensionAttributes->getFbAdditionalData());
                    if(array_key_exists("external_invoices", $additionalData)){
                        $additionalData["external_invoices"][$invoice_data->getIncrementId()]=$invoice;
                    }else{
                        $additionalData["external_invoices"]=array($invoice_data->getIncrementId() => $invoice);
                    }
                }
            }
            else{
                $orderExtensionAttributes=$this->_orderExtensionFactory->create();
            }
            $orderExtensionAttributes->setFbAdditionalData(array($additionalData));
            $order->setExtensionAttributes($orderExtensionAttributes);
            $order->addStatusHistoryComment(__("External document has been saved: %1 %2", $documentType, $invoiceIncrementId), "invoiced");
            if($original_status=="sent_to_logistics"){
                $order->addStatusHistoryComment(__("Order has been sent to logistics"), "sent_to_logistics");
            }
            $this->_orderRepository->save($order);
            $this->_response->setStatus("success");
            $this->_response->setMessage(__("External document has been saved"));
            $this->_response->setFlow("create_document");            
            $this->_response->setOrderIncrementId($order->getIncrementId());            
            return $this->_response;
        }catch(Exception $e){
            $this->_logger->critical($e->getMessage());
            throw $e;
        }
    }
}

